@@ -542,7 +542,8 @@ srl_dump_ivuv(pTHX_ srl_encoder_t *enc, SV *src)
      *      we're just wasting some space. */
     /* TODO optimize! */
 
-    if (SvIOK_UV(src) || SvIV(src) >= 0) { /* FIXME find a way to express this without repeated SvIV/SvUV */
+    /* FIXME find a way to express the condition without repeated SvIV/SvUV */
+    if (expect_true( SvIOK_UV(src) || SvIV(src) >= 0 )) {
         const UV num = SvUV(src); /* FIXME is SvUV_nomg good enough because of the GET magic in dump_sv? SvUVX after having checked the flags? */
         if (num < 16) {
             /* encodable as POS */
@@ -580,7 +581,7 @@ srl_dump_classname(pTHX_ srl_encoder_t *enc, SV *src, int use_freeze)
     if (oldoffset != 0) {
         /* Issue COPY instead of literal class name string */
         srl_buf_cat_varint(aTHX_ enc,
-                                 UNLIKELY(use_freeze) ? SRL_HDR_OBJECTV_FREEZE : SRL_HDR_OBJECTV,
+                                 expect_false(use_freeze) ? SRL_HDR_OBJECTV_FREEZE : SRL_HDR_OBJECTV,
                                  (UV)oldoffset);
     }
     else {
@@ -594,7 +595,7 @@ srl_dump_classname(pTHX_ srl_encoder_t *enc, SV *src, int use_freeze)
          * At least, we can safely use the same PTABLE to store the ptrs to hashkeys since
          * the set of pointers will never collide.
          * /me bows to Yves for the delightfully evil hack. */
-        srl_buf_cat_char(enc, UNLIKELY(use_freeze) ? SRL_HDR_OBJECT_FREEZE : SRL_HDR_OBJECT);
+        srl_buf_cat_char(enc, expect_false(use_freeze) ? SRL_HDR_OBJECT_FREEZE : SRL_HDR_OBJECT);
 
         /* remember current offset before advancing it */
         PTABLE_store(string_seenhash, (void *)stash, (void *)BODY_POS_OFS(enc->buf));
@@ -1175,12 +1176,12 @@ srl_dump_object(pTHX_ srl_encoder_t *enc, SV *referent, SV *obj)
     /* FIXME reuse/ref/... should INCLUDE the bless stuff. */
 
     /* Check for FREEZE support */
-    if (UNLIKELY( SRL_ENC_HAVE_OPTION(enc, SRL_F_ENABLE_FREEZE_SUPPORT) )) {
+    if (expect_false( SRL_ENC_HAVE_OPTION(enc, SRL_F_ENABLE_FREEZE_SUPPORT) )) {
         HV *stash = SvSTASH(referent);
         assert(stash != NULL);
         GV *method = gv_fetchmethod_autoload(stash, "FREEZE", 0);
 
-        if (UNLIKELY( method != NULL )) {
+        if (expect_false( method != NULL )) {
             int count;
             dSP;
             ENTER;
@@ -1196,7 +1197,7 @@ srl_dump_object(pTHX_ srl_encoder_t *enc, SV *referent, SV *obj)
             /* TODO explore method lookup caching */
             SPAGAIN;
 
-            if (LIKELY( count == 1 )) {
+            if (expect_true( count == 1 )) {
                 object_content = POPs;
                 SvREFCNT_inc(object_content);
             }
@@ -1222,7 +1223,7 @@ srl_dump_object(pTHX_ srl_encoder_t *enc, SV *referent, SV *obj)
         }
     }
     /* If we have SRL_F_NO_BLESS_OBJECTS, then do nothing. Otherwise, emit OBJECT* tag */
-    else if (LIKELY( !SRL_ENC_HAVE_OPTION(enc, SRL_F_NO_BLESS_OBJECTS) )) {
+    else if (expect_true( !SRL_ENC_HAVE_OPTION(enc, SRL_F_NO_BLESS_OBJECTS) )) {
         srl_dump_classname(aTHX_ enc, referent, 0); /* 0 == no freeze call */
         object_content = obj;
     }
@@ -1252,7 +1253,7 @@ srl_dump_sv(pTHX_ srl_encoder_t *enc, SV *src)
     SSize_t ref_rewrite_pos= 0;      /* preserved between loops - note SSize_t is a perl define */
     assert(src);
 
-    if (++enc->recursion_depth == enc->max_recursion_depth) {
+    if (expect_false( ++enc->recursion_depth == enc->max_recursion_depth )) {
         croak("Hit maximum recursion depth (%lu), aborting serialization",
               (unsigned long)enc->max_recursion_depth);
     }
@@ -1276,7 +1277,7 @@ redo_dump:
         if (DEBUGHACK) warn("backreferences %p", src);
     }
 #endif
-    if ( mg || backrefs ) {
+    if (expect_false( mg || backrefs )) {
         PTABLE_t *weak_seenhash= SRL_GET_WEAK_SEENHASH(enc);
         PTABLE_ENTRY_t *pe= PTABLE_find(weak_seenhash, src);
         if (!pe) {
@@ -1329,7 +1330,7 @@ redo_dump:
             PTABLE_store(ref_seenhash, src, (void *)BODY_POS_OFS(enc->buf));
         }
     }
-    if (weakref_ofs != 0) {
+    if (expect_false( weakref_ofs != 0 )) {
         sv_dump(src);
         croak("Corrupted weakref? weakref_ofs=0 (this should not happen)");
     }
@@ -1373,7 +1374,7 @@ redo_dump:
             assert(referent);
         }
 #endif
-        if (SvWEAKREF(src)) {
+        if (expect_false( SvWEAKREF(src) )) {
             if (DEBUGHACK) warn("Is weakref %p", src);
             weakref_ofs= BODY_POS_OFS(enc->buf);
             srl_buf_cat_char(enc, SRL_HDR_WEAKEN);
